Package: BayesPocket
Type: Package
Title: Bayesian Causal Inference for Periodontal Diseases in
        Longitudinal Studies
Version: 0.1.0
Authors@R: c(
    person("Qingyang", "Liu", email = "rh8liuqy@gmail.com", role = c("aut", "cre"), comment = c(ORCID = "0000-0003-3265-6330")),
    person("Debdeep", "Pati", email = "dpati2@wisc.edu", role = c("aut")),
    person("Yang", "Ni", email = "yang.ni@austin.utexas.edu", role = c("aut")),
    person("Dipankar", "Bandyopadhyay", email = "dbandyop@vcu.edu", role = "aut")
  )
Description: Implements the Mixed Treatment-State Causal Model (MTSCM), a Bayesian 
    framework for estimating causal effects of clinical interventions on bounded 
    continuous outcomes in longitudinal observational studies with irregular visits. 
    The methodology is specifically designed for periodontal disease research, where 
    discrete treatments and continuous disease states (e.g., proportion of periodontal 
    pockets exceeding 3 mm) reciprocally influence one another under dynamic feedback. 
    The package integrates a double-censored Tobit likelihood to handle boundary mass 
    at zero and one, subject-specific random effects to capture within-subject 
    correlation, and flexible tree-based ensemble priors (standard BART and Soft BART) 
    to model complex nonlinear interactions without parametric restrictions. Causal 
    identification is established under the potential outcomes framework via the 
    G-computation formula, with key estimands including the Mixed Average Potential 
    Outcome (MAPO) and the Mixed Probability of Disease Resolution (MPDR). The 
    package provides functions for model fitting, posterior inference, and causal estimand 
    estimation.
License: GPL (>= 3)
Encoding: UTF-8
RoxygenNote: 7.3.2
Imports: stats (>= 4.4.2), GIGrvg(>= 0.8), truncnorm(>= 1.0-9),
        progress(>= 1.2.3), stochtree(>= 0.1.1), SoftBart(>= 1.0.3),
        parallel(>= 4.4.2), pbmcapply(>= 1.5.1)
Depends: R (>= 3.5)
NeedsCompilation: no
Packaged: 2026-05-08 20:31:34 UTC; kevin_liu
Author: Qingyang Liu [aut, cre] (ORCID:
    <https://orcid.org/0000-0003-3265-6330>),
  Debdeep Pati [aut],
  Yang Ni [aut],
  Dipankar Bandyopadhyay [aut]
Maintainer: Qingyang Liu <rh8liuqy@gmail.com>
Repository: CRAN
Date/Publication: 2026-05-13 08:10:22 UTC
Built: R 4.5.2; ; 2026-05-13 11:23:31 UTC; unix

Package: BayesPocket
Type: Package
Title: Bayesian Causal Inference for Periodontal Diseases in
        Longitudinal Studies
Version: 0.1.0
Authors@R: c(
    person("Qingyang", "Liu", email = "rh8liuqy@gmail.com", role = c("aut", "cre"), comment = c(ORCID = "0000-0003-3265-6330")),
    person("Debdeep", "Pati", email = "dpati2@wisc.edu", role = c("aut")),
    person("Yang", "Ni", email = "yang.ni@austin.utexas.edu", role = c("aut")),
    person("Dipankar", "Bandyopadhyay", email = "dbandyop@vcu.edu", role = "aut")
  )
Description: Implements the Mixed Treatment-State Causal Model (MTSCM), a Bayesian 
    framework for estimating causal effects of clinical interventions on bounded 
    continuous outcomes in longitudinal observational studies with irregular visits. 
    The methodology is specifically designed for periodontal disease research, where 
    discrete treatments and continuous disease states (e.g., proportion of periodontal 
    pockets exceeding 3 mm) reciprocally influence one another under dynamic feedback. 
    The package integrates a double-censored Tobit likelihood to handle boundary mass 
    at zero and one, subject-specific random effects to capture within-subject 
    correlation, and flexible tree-based ensemble priors (standard BART and Soft BART) 
    to model complex nonlinear interactions without parametric restrictions. Causal 
    identification is established under the potential outcomes framework via the 
    G-computation formula, with key estimands including the Mixed Average Potential 
    Outcome (MAPO) and the Mixed Probability of Disease Resolution (MPDR). The 
    package provides functions for model fitting, posterior inference, and causal estimand 
    estimation.
License: GPL (>= 3)
Encoding: UTF-8
RoxygenNote: 7.3.2
Imports: stats (>= 4.4.2), GIGrvg(>= 0.8), truncnorm(>= 1.0-9),
        progress(>= 1.2.3), stochtree(>= 0.1.1), SoftBart(>= 1.0.3),
        parallel(>= 4.4.2), pbmcapply(>= 1.5.1)
Depends: R (>= 3.5)
NeedsCompilation: no
Packaged: 2026-05-08 20:31:34 UTC; kevin_liu
Author: Qingyang Liu [aut, cre] (ORCID:
    <https://orcid.org/0000-0003-3265-6330>),
  Debdeep Pati [aut],
  Yang Ni [aut],
  Dipankar Bandyopadhyay [aut]
Maintainer: Qingyang Liu <rh8liuqy@gmail.com>
Repository: CRAN
Date/Publication: 2026-05-13 08:10:22 UTC
Built: R 4.6.0; ; 2026-05-13 08:54:34 UTC; unix

# Generated by roxygen2: do not edit by hand

export(causal_estimand_inference)
export(causal_inference_model)
export(data_generation)
importFrom(GIGrvg,rgig)
importFrom(SoftBart,Hypers)
importFrom(SoftBart,MakeForest)
importFrom(SoftBart,Opts)
importFrom(parallel,mclapply)
importFrom(pbmcapply,pbmclapply)
importFrom(progress,progress_bar)
importFrom(stats,dnorm)
importFrom(stats,integrate)
importFrom(stats,model.matrix)
importFrom(stats,pnorm)
importFrom(stats,quantile)
importFrom(stats,rgamma)
importFrom(stats,rnorm)
importFrom(stats,rpois)
importFrom(stochtree,createCppRNG)
importFrom(stochtree,createForest)
importFrom(stochtree,createForestDataset)
importFrom(stochtree,createForestModel)
importFrom(stochtree,createForestModelConfig)
importFrom(stochtree,createForestSamples)
importFrom(stochtree,createGlobalModelConfig)
importFrom(stochtree,createOutcome)
importFrom(truncnorm,rtruncnorm)

# data generation ---------------------------------------------------------

df1 <- data_generation(random_seed = 100,
                       N = 100,
                       sigma = 0.2,
                       sigma_u = 0.1)

# draw samples from the posterior distribution ----------------------------

inference_output <- causal_inference_model(df = df1,
                                           y_name = "current_value",
                                           continuous_name = c("previous_value",
                                                               "confounder"),
                                           categorical_name = c("treatment"),
                                           treatment_name = "treatment",
                                           previous_status_name = "previous_value",
                                           subjectID_name = "subjectID",
                                           num_warmup = 2,
                                           num_samples = 2,
                                           model_type = "Tobit-XBART",
                                           thin = 1,
                                           L = 5,
                                           alpha = 0.95,
                                           beta = 1.25,
                                           leaf_model_scale = 0.3/5,
                                           cutpoint_grid_size = 100,
                                           max_depth = 10,
                                           credible_interval_level = 0.95,
                                           random_seed = 100,
                                           calculate_causal_estimand = FALSE,
                                           previous_status_grid_size = 2)

# calculate the causal estimand over a grid of previous status values -------

outcome_model_results <- inference_output$outcome_model_results

inference_results <- causal_estimand_inference(outcome_model_results = outcome_model_results,
                 df = inference_output$df,
                 continuous_name = inference_output$continuous_name,
                 categorical_name = inference_output$categorical_name,
                 treatment_name = inference_output$treatment_name,
                 treatment_value = factor("TREATMENT A",
                                          levels = levels(df1$treatment)),
                 previous_status_name = inference_output$previous_status_name,
                 credible_interval_level = 0.95,
                 num_of_grids = 2) # Example uses a small 2-point grid

# View the newly calculated closed-form causal estimands ------------------

# 1. Print results for the first grid point
cat("--- Results for Grid Point 1 ---\n")
print(inference_results[[1]]$mapo_summary)
print(inference_results[[1]]$mpdr_summary)

# 2. Print results for the second grid point
cat("\n--- Results for Grid Point 2 ---\n")
print(inference_results[[2]]$mapo_summary)
print(inference_results[[2]]$mpdr_summary)

df1 <- data_generation(random_seed = 100,
                       N = 100,
                       sigma = 0.2,
                       sigma_u = 0.1)

inference_output <- causal_inference_model(df = df1,
                                           y_name = "current_value",
                                           continuous_name = c("previous_value",
                                                               "confounder"),
                                           categorical_name = c("treatment"),
                                           treatment_name = "treatment",
                                           previous_status_name = "previous_value",
                                           subjectID_name = "subjectID",
                                           num_warmup = 2,
                                           num_samples = 2,
                                           model_type = "Tobit-XBART",
                                           thin = 1,
                                           L = 5,
                                           alpha = 0.95,
                                           beta = 1.25,
                                           leaf_model_scale = 0.3/5,
                                           cutpoint_grid_size = 100,
                                           max_depth = 10,
                                           credible_interval_level = 0.95,
                                           random_seed = 100,
                                           calculate_causal_estimand = FALSE,
                                           previous_status_grid_size = 2)

doubly_censored_expectation <- function(theta,sigma) {
  pM1 <- theta*(pnorm((1.0 - theta)/sigma) - pnorm((0.0 - theta)/sigma))
  pM2 <- sigma*(dnorm((0.0 - theta)/sigma) - dnorm((1.0 - theta)/sigma))
  pR <- 1.0 * (1.0 - pnorm((1.0 - theta)/sigma))
  output <- pM1 + pM2 + pR
  return(output)
}

# Create group_info by repeating group numbers based on ni_vector values
ni_to_group_info <- function(ni_vector) {
  group_info <- unlist(lapply(seq_along(ni_vector), function(i) {
    rep(i, ni_vector[i])
  }))

  return(group_info)
}

# based on A Simple Sampler for the Horseshoe Estimator by Enes Makalic and Daniel F. Schmidt
# Robust version: Prevents computationally singular errors
horseshoe_iteration <- function(y, X, sigma2, lambda2, tau2, nu, xi) {

  n <- nrow(X)
  p <- ncol(X)

  # ---------------------------------------------------------
  # 1. Update beta
  # ---------------------------------------------------------
  # Clamp variances to prevent 1/0 creating Inf in the matrix
  effective_variances <- pmax(tau2 * lambda2, 1e-12)
  Lambda_star_inv <- diag(1 / effective_variances, nrow = p, ncol = p)

  A <- crossprod(X) + Lambda_star_inv

  # Add microscopic jitter to diagonal to guarantee matrix condition
  diag(A) <- diag(A) + 1e-10

  # Use Cholesky decomposition (vastly more stable than solve)
  R <- chol(A)
  A_inv <- chol2inv(R)

  # Calculate mean for beta
  mu_beta <- A_inv %*% crossprod(X, y)

  # Sample beta using the Cholesky factor R directly
  # backsolve(R, z) is mathematically identical to t(chol(A_inv)) %*% z but much faster
  z <- rnorm(p)
  beta <- as.numeric(mu_beta + sqrt(sigma2) * backsolve(R, z))

  # ---------------------------------------------------------
  # 2. Update lambda2_j (local hypervariances)
  # ---------------------------------------------------------
  shape_lambda <- 1
  safe_tau2 <- max(tau2, 1e-12) # Prevent division by zero
  rate_lambda <- 1/nu + (beta^2) / (2 * safe_tau2 * sigma2)

  lambda2 <- 1 / rgamma(p, shape = shape_lambda, rate = rate_lambda)
  lambda2 <- pmax(lambda2, 1e-12) # Clamp output

  # ---------------------------------------------------------
  # 3. Update tau2 (global hypervariance)
  # ---------------------------------------------------------
  shape_tau <- (p + 1) / 2
  safe_lambda2 <- pmax(lambda2, 1e-12) # Prevent division by zero
  rate_tau <- 1/xi + sum(beta^2 / safe_lambda2) / (2 * sigma2)

  tau2 <- 1 / rgamma(1, shape = shape_tau, rate = rate_tau)
  tau2 <- max(tau2, 1e-12) # Clamp output

  # ---------------------------------------------------------
  # 4. Update nu_j (auxiliary variables)
  # ---------------------------------------------------------
  shape_nu <- 1
  rate_nu <- 1 + (1 / safe_lambda2)
  nu <- 1 / rgamma(p, shape = shape_nu, rate = rate_nu)

  # ---------------------------------------------------------
  # 5. Update xi (auxiliary variable)
  # ---------------------------------------------------------
  shape_xi <- 1
  rate_xi <- 1 + (1 / safe_tau2)
  xi <- 1 / rgamma(1, shape = shape_xi, rate = rate_xi)

  # Return updated parameters
  return(list(
    beta = beta,
    lambda2 = lambda2,
    tau2 = tau2,
    nu = nu,
    xi = xi
  ))
}

#' @title Calculate Causal Estimands for a Single Treatment and Previous Status
#'
#' @description
#' Calculate the causal estimands (MAPO and MPDR) for a single given treatment value and a single previous status value.
#'
#' @details
#' For comprehensive details regarding the underlying framework, methodology, and the main model fitting procedure, please refer to \code{\link{causal_inference_model}}.
#'
#' @param outcome_model_results the results from the outcome model. The datatype is \code{list}.
#' @param df the input dataframe. The datatype is \code{data.frame}.
#' @param continuous_name the name of continuous predictors. The datatype is \code{character}.
#' @param categorical_name the name of categorical predictors. The datatype is \code{character}.
#' @param treatment_name the name of the treatment predictor. The datatype is \code{character}.
#' @param treatment_value the value of the treatment variable. The datatype is \code{factor}.
#' @param previous_status_name the name of the variable that represents the previous status of a subject. The datatype is \code{character}.
#' @param previous_status_value the value of the previous status. It should be a value between [0,1]. The datatype is \code{double}.
#' @param credible_interval_level the nominal level of credible intervals of the causal estimand. The datatype is \code{double}.
#'
#' @seealso \code{\link{causal_inference_model}}
#'
#' @returns a list containing \code{data.frame}s of calculated causal estimands (MAPO and MPDR).
#' @noRd
causal_estimand_single_value <- function(outcome_model_results,
                                         df,
                                         continuous_name,
                                         categorical_name,
                                         treatment_name,
                                         treatment_value,
                                         previous_status_name,
                                         previous_status_value,
                                         credible_interval_level = 0.95) {

  num_samples <- nrow(outcome_model_results$fx_output)
  num_warmup <- outcome_model_results$num_warmup
  thin <- outcome_model_results$thin
  model_type <- outcome_model_results$model_type

  # if df is a tibble, convert it as dataframe
  df <- as.data.frame(df)
  # define X_continuous
  X_continuous <- as.data.frame(df[continuous_name])
  # given previous status value
  X_continuous[,previous_status_name] <- previous_status_value
  X_continuous <- as.matrix(X_continuous)
  # define X_categorical
  X_categorical <- as.data.frame(df[categorical_name])
  # given treatment type
  X_categorical[,treatment_name] <- treatment_value

  # create dataset for prediction
  if (model_type %in% c("Tobit-XBART","N-XBART")) {
    # convert to numeric
    X_categorical <- sapply(X_categorical, as.numeric)
    Xall <- cbind(X_continuous,X_categorical)
    forest_dataset <- createForestDataset(Xall)
  } else if (model_type %in% c("Tobit-SBART","N-SBART")) {
    X_categorical <- model.matrix(~ . - 1, data = X_categorical)
    Xall <- cbind(X_continuous,X_categorical)
  } else if (model_type %in% c("Tobit-LH","N-LH")) {
    Xall <- generate_design_matrix(X_continuous = as.data.frame(X_continuous),
                                   X_categorical = X_categorical)
  }

  # Initialize the causal estimand output utilizing closed forms
  causal_estimand_output <- lapply(1:num_samples, function(i) {

    # Calculate the correct index in the full, unthinned MCMC chain
    actual_idx <- num_warmup + (i * thin)

    if (model_type %in% c("Tobit-XBART","N-XBART")) {
      fx_value <- outcome_model_results$forest_samples$predict_raw_single_forest(
        forest_dataset = forest_dataset,
        forest_num = actual_idx - 1
      )
    } else if (model_type %in% c("Tobit-SBART","N-SBART")) {
      fx_value <- outcome_model_results$softforest$predict_iteration(X = Xall,
                                                                     i = actual_idx - 1)
    } else if(model_type %in% c("Tobit-LH","N-LH")) {
      fx_value <- Xall %*% outcome_model_results$beta_output[i,]
    }
    fx_value <- as.numeric(fx_value)

    global_error_variance_value <- outcome_model_results$global_error_variance_output[i]
    random_effects_variance_value <- outcome_model_results$random_effects_variance_output[i]

    if (model_type %in% c("Tobit-XBART", "Tobit-SBART", "Tobit-LH")) {
      # Closed-form parameters
      sigma_sq <- global_error_variance_value
      sigma_star <- sqrt(sigma_sq + random_effects_variance_value)

      # Convolution terms for MAPO and MPDR
      term_1 <- pnorm((1 - fx_value) / sigma_star)
      term_2 <- pnorm(-fx_value / sigma_star)
      density_1 <- dnorm(-fx_value / sigma_star)
      density_2 <- dnorm((1 - fx_value) / sigma_star)

      # MAPO (Continuous Expectation): G-computation via mean() over all observations
      mapo_values <- fx_value * (term_1 - term_2) +
        sigma_star * (density_1 - density_2) +
        (1 - term_1)
      mapo_value <- mean(mapo_values)

      # MPDR (Probability of Disease Resolution): G-computation via mean() over all observations
      mpdr_values <- term_2
      mpdr_value <- mean(mpdr_values)

    } else if (model_type %in% c("N-XBART", "N-SBART", "N-LH")) {
      # for normal likelihood, the E(y) = f(X). Just take average of f(X).
      mapo_value <- mean(fx_value)
      mpdr_value <- NA # MPDR is typically defined via the Tobit boundary mass
    }

    output <- list(mapo_value = mapo_value,
                   mpdr_value = mpdr_value,
                   fx_value = fx_value)

    return(output)
  })

  # Convert the list to numeric vectors
  mapo_value <- sapply(causal_estimand_output, function(x) {x$mapo_value})
  mpdr_value <- sapply(causal_estimand_output, function(x) {x$mpdr_value})
  fx_value <- do.call("rbind", lapply(causal_estimand_output, function(x) {x$fx_value}))

  # Summarize MAPO
  df_mapo_output <- data.frame(treatment = treatment_value,
                               previous_status = previous_status_value,
                               mapo_mean = mean(mapo_value),
                               mapo_lb = quantile(mapo_value , probs = (1-credible_interval_level)/2),
                               mapo_ub = quantile(mapo_value , probs = 1 - (1-credible_interval_level)/2))
  row.names(df_mapo_output) <- NULL

  # Summarize MPDR
  df_mpdr_output <- data.frame(treatment = treatment_value,
                               previous_status = previous_status_value,
                               mpdr_mean = mean(mpdr_value, na.rm = TRUE),
                               mpdr_lb = quantile(mpdr_value, probs = (1-credible_interval_level)/2, na.rm = TRUE),
                               mpdr_ub = quantile(mpdr_value, probs = 1 - (1-credible_interval_level)/2, na.rm = TRUE))
  row.names(df_mpdr_output) <- NULL

  output <- list(treatment = treatment_value,
                 previous_status = previous_status_value,
                 credible_interval_level = credible_interval_level,
                 mapo_summary = df_mapo_output,
                 mpdr_summary = df_mpdr_output,
                 mapo_value = mapo_value,
                 mpdr_value = mpdr_value,
                 fx_value = fx_value)

  return(output)
}


#' @title Iterate Causal Estimand Calculations Over a Grid
#'
#' @description
#' Wrapper to iterate causal estimand calculations over a grid of previous status values.
#'
#' @details
#' This function evaluates the causal estimands (such as MAPO and MPDR) across a specified grid of values for the previous disease state. For comprehensive details regarding the underlying framework, methodology, and the main model fitting procedure, please refer to \code{\link{causal_inference_model}}.
#'
#' @param outcome_model_results the results from the outcome model. The datatype is \code{list}.
#' @param df the input dataframe. The datatype is \code{data.frame}.
#' @param continuous_name the name of continuous predictors. The datatype is \code{character}.
#' @param categorical_name the name of categorical predictors. The datatype is \code{character}.
#' @param treatment_name the name of the treatment predictor. The datatype is \code{character}.
#' @param treatment_value the value of the treatment variable. The datatype is \code{factor}.
#' @param previous_status_name the name of the variable that represents previous status. The datatype is \code{character}.
#' @param credible_interval_level the nominal level of credible intervals of causal estimand. The datatype is \code{double}.
#' @param num_of_grids the number of grid points in [0,1] to evaluate previous status on. The datatype is \code{integer}.
#'
#' @seealso \code{\link{causal_inference_model}}
#'
#' @returns a \code{list} of causal estimands calculated across the evaluation grid.
#' @export
#'
#' @example inst/examples/causal_estimand_inference.r
#'
causal_estimand_inference <- function(outcome_model_results,
                                      df,
                                      continuous_name,
                                      categorical_name,
                                      treatment_name,
                                      treatment_value,
                                      previous_status_name,
                                      credible_interval_level = 0.95,
                                      num_of_grids = 50) {

  # Replaced parallel processing with a fast closed-form standard lapply
  causal_estimand_output <- lapply(X = seq(0, 1, length.out = num_of_grids),
                                   FUN = function(x) {
                                     causal_estimand_single_value(outcome_model_results = outcome_model_results,
                                                                  df = df,
                                                                  continuous_name = continuous_name,
                                                                  categorical_name = categorical_name,
                                                                  treatment_name = treatment_name,
                                                                  treatment_value = treatment_value,
                                                                  previous_status_name = previous_status_name,
                                                                  previous_status_value = x,
                                                                  credible_interval_level = credible_interval_level)
                                   })

  return(causal_estimand_output)
}

update_U <- function(Z,fixed_effects,ni_vector,group_info,
                     global_error_variance,
                     random_effects_variance) {
  Zstar <- Z - fixed_effects
  sum_Zstar_ij <- as.numeric(tapply(Zstar, group_info, sum))
  loc <- (random_effects_variance) * sum_Zstar_ij / (ni_vector * random_effects_variance + global_error_variance)
  variance <- (global_error_variance * random_effects_variance) / (ni_vector*random_effects_variance + global_error_variance)
  N <- length(ni_vector)
  output <- rnorm(n = N) * sqrt(variance) + loc
  return(output)
}

update_global_error_variance <- function(Z,fixed_effects,random_effects) {
  # prior is Gamma(1,1) - shape, rate
  lambda <- 1.0 - 0.5*length(Z) #p
  psi <- 1.0*2 #a


  residuals_value <- Z - fixed_effects - random_effects
  SSR <- sum(residuals_value^2)
  chi <- SSR #b

  output <- rgig(n = 1, lambda = lambda, chi = chi, psi = psi)
  return(output)
}

update_random_effects_variance <- function(U_value, ni_vector) {
  # prior is Gamma(1,1) - shape, rate
  lambda <- 1.0 - 0.5*length(ni_vector) #p
  psi <- 1.0*2 #a
  chi <- sum(U_value^2) #b

  output <- rgig(n = 1, lambda = lambda, chi = chi, psi = psi)
  return(output)
}

#' @title Bayesian Mixed Treatment-State Causal Model (MTSCM)
#'
#' @description
#' Fits a Bayesian Mixed Treatment-State Causal Model (MTSCM) tailored for longitudinal settings with irregular visits.
#' This model is specifically designed for bounded continuous outcomes with mass at both boundaries, such as the proportion of periodontal pockets exceeding 3 mm.
#'
#' @details
#' \strong{Outcome Model Description:}
#' The MTSCM utilizes a double-censored Tobit regression structure with subject-level random effects to capture within-subject correlation.
#' It uses tree-based ensemble priors (encompassing standard BART and Soft BART) to flexibly model complex, non-linear interactions without parametric restrictions.
#' Using a latent Gaussian variable \eqn{Z_{i,j+1}}, the data generating process for the outcome \eqn{Y_{i,j+1}} is formulated as:
#' \deqn{Z_{i,j+1} = f(A_{ij}, Y_{ij}, \mathbf{X}_{ij}) + U_i + \epsilon_{ij}, \quad \epsilon_{ij} \sim N(0, \sigma^2).}
#' \deqn{Y_{i,j+1} = \begin{cases} 0 & \text{if } Z_{i,j+1} \leq 0, \\ Z_{i,j+1} & \text{if } 0 < Z_{i,j+1} < 1, \\ 1 & \text{if } Z_{i,j+1} \geq 1. \end{cases}}
#'
#' \strong{Causal Estimand Description:}
#' The framework estimates causal effects using the G-computation formula to marginalize conditional expectations over the covariate distribution.
#'
#' \strong{Mixed Average Potential Outcome (MAPO):}
#' The MAPO represents the expected potential outcome across the population for a given treatment \eqn{a} and previous continuous disease state \eqn{y^{\star}}.
#' It is defined mathematically as:
#' \deqn{\theta(a,y^{\star}) = \frac{\sum_{i=1}^{N} \sum_{j=0}^{n_i - 1} \mathbb{E}(Y_{i,j+1} \mid A_{ij} = a, Y_{ij} = y^{\star}, \mathbf{X}_{ij} = \mathbf{x}_{ij})}{\sum_{i=1}^{N} n_{i}}.}
#'
#' \strong{Mixed Probability of Disease Resolution (MPDR):}
#' The MPDR estimates the population-level probability of achieving a zero disease burden under a specific treatment \eqn{a} and previous disease state \eqn{y^{\star}}.
#' It is computed as:
#' \deqn{\theta^{(0)}(a,y^{\star}) = \frac{\sum_{i=1}^{N} \sum_{j=0}^{n_i - 1} \mathbb{P}(Y_{i,j+1} = 0 \mid A_{ij} = a, Y_{ij} = y^{\star}, \mathbf{X}_{ij} = \mathbf{x}_{ij})}{\sum_{i=1}^{N} n_{i}}.}
#'
#' @param df the input dataframe. The datatype is \code{data.frame}.
#' @param y_name the name of the response variable. The datatype is \code{character}.
#' @param continuous_name the name of continuous predictors. The datatype is \code{character}.
#' @param categorical_name the name of categorical predictors. The datatype is \code{character}.
#' @param treatment_name the name of the treatment predictor. The datatype is \code{character}.
#' @param previous_status_name the name of variable that represents previous status of a subject. The datatype is \code{character}.
#' @param subjectID_name the name of variable that represents subjectID. The datatype is \code{character}.
#' @param num_warmup the number of warmup iterations. The datatype is \code{integer}.
#' @param num_samples the number of post-warmup iterations. The datatype is \code{integer}.
#' @param model_type the type of causal inference models. It must be one of \code{"Tobit-XBART"}, \code{"Tobit-SBART"}, \code{"Tobit-LH"}, \code{"N-XBART"}, \code{"N-SBART"}, or \code{"N-LH"}.
#' @param thin the period between saved samples. This should typically be left at its default (no thinning) unless memory is a problem. The datatype is \code{integer}.
#' @param L the number of trees. The datatype is \code{integer}.
#' @param alpha the tree prior parameters. \code{alpha*(1+depth)^(-beta)} represents the prior probability of splitting at one of the cutpoints. Check equation (4) and related descriptions from "Stochastic Tree Ensembles for Regularized Nonlinear Regression" for more details. The datatype is \code{double}.
#' @param beta the tree prior parameters. \code{alpha*(1+depth)^(-beta)} represents the prior probability of splitting at one of the cutpoints. Check equation (4) and related descriptions from "Stochastic Tree Ensembles for Regularized Nonlinear Regression" for more details. The datatype is \code{double}.
#' @param leaf_model_scale the prior variance on leaf mean equals to \code{leaf_model_scale/L}. The datatype is \code{double}.
#' @param cutpoint_grid_size the number of cutoff points in XBART. The datatype is \code{integer}.
#' @param max_depth the maximum depth of tree allowed. The datatype is \code{integer}.
#' @param credible_interval_level the nominal level of credible intervals of causal estimand. The datatype is \code{double}.
#' @param print_progress whether print progress bar or not. The datatype is \code{boolean}.
#' @param random_seed the random seed of the MCMC sampler. The datatype is \code{integer}.
#' @param calculate_causal_estimand calculate causal estimand or not. The datatype is \code{boolean}.
#' @param previous_status_grid_size the number of cutoff points of previous status for causal estimands. The datatype is \code{integer}.
#' @returns a list containing posterior inference of causal estimands.
#' @export
#'
#' @example inst/examples/main.r
causal_inference_model <- function(df,
                                   y_name,
                                   continuous_name,
                                   categorical_name,
                                   treatment_name,
                                   previous_status_name,
                                   subjectID_name,
                                   num_warmup,
                                   num_samples,
                                   model_type,
                                   thin = 1,
                                   L = 50,
                                   alpha = 0.95,
                                   beta = 1.25,
                                   leaf_model_scale = 0.3/50,
                                   cutpoint_grid_size = 100,
                                   max_depth = 10,
                                   credible_interval_level = 0.95,
                                   print_progress = TRUE,
                                   random_seed = 100,
                                   calculate_causal_estimand = FALSE,
                                   previous_status_grid_size = 100) {

  # Type checking for all parameters
  if (!is.data.frame(df)) {
    stop("df must be a data frame")
  }

  # avoid problem caused by tibble
  df <- as.data.frame(df)

  if (!is.character(y_name) || length(y_name) != 1) {
    stop("y_name must be a single character string")
  }

  if (!is.character(continuous_name)) {
    stop("continuous_name must be a character vector")
  }

  if (!is.character(categorical_name)) {
    stop("categorical_name must be a character vector")
  }

  if (!is.character(treatment_name) || length(treatment_name) != 1) {
    stop("treatment_name must be a single character string")
  }

  if (!is.character(previous_status_name) || length(previous_status_name) != 1) {
    stop("previous_status_name must be a single character string")
  }

  if (!is.character(subjectID_name) || length(subjectID_name) != 1) {
    stop("subjectID_name must be a single character string")
  }

  if (!is.numeric(num_warmup) || num_warmup <= 0 || num_warmup != as.integer(num_warmup)) {
    stop("num_warmup must be a positive integer")
  }

  if (!is.numeric(num_samples) || num_samples <= 0 || num_samples != as.integer(num_samples)) {
    stop("num_samples must be a positive integer")
  }

  if (!is.character(model_type) || ! model_type %in% c("Tobit-XBART",
                                                       "Tobit-SBART",
                                                       "Tobit-LH",
                                                       "N-XBART",
                                                       "N-SBART",
                                                       "N-LH")) {
    stop("model_type must be a string and one of Tobit-XBART, Tobit-SBART, Tobit-LH, N-XBART, N-SBART, N-LH")
  }

  if (!is.numeric(thin) || thin < 1 || thin != as.integer(thin)) {
    stop("thin must be an integer at least 1")
  }

  # Additional safe checks for thin and num_samples relationship
  if (num_samples <= thin) {
    stop("num_samples must be larger than thin")
  }

  if (num_samples %% thin != 0) {
    stop("num_samples must be divisible by thin (ratio must be an integer)")
  }

  if (!is.numeric(L) || L <= 0 || L != as.integer(L)) {
    stop("L must be a positive integer")
  }

  if (!is.numeric(alpha) || alpha <= 0) {
    stop("alpha must be a positive numeric value")
  }

  if (!is.numeric(beta) || beta <= 0) {
    stop("beta must be a positive numeric value")
  }

  if (!is.numeric(leaf_model_scale) || leaf_model_scale <= 0) {
    stop("leaf_model_scale must be a positive numeric value")
  }

  if (!is.numeric(cutpoint_grid_size) || cutpoint_grid_size <= 0 || cutpoint_grid_size != as.integer(cutpoint_grid_size)) {
    stop("cutpoint_grid_size must be a positive integer")
  }

  if (!is.numeric(max_depth) || max_depth <= 0 || max_depth != as.integer(max_depth)) {
    stop("max_depth must be a positive integer")
  }

  if (!is.numeric(credible_interval_level) || credible_interval_level <= 0 || credible_interval_level >= 1) {
    stop("credible_interval_level must be a numeric value between 0 and 1")
  }

  if (!is.logical(print_progress)) {
    stop("print_progress must be a logical value (TRUE or FALSE)")
  }

  if (!is.numeric(random_seed) || random_seed != as.integer(random_seed)) {
    stop("random_seed must be an integer")
  }

  if (!is.logical(calculate_causal_estimand)) {
    stop("calculate_causal_estimand must be a logical value (TRUE or FALSE)")
  }

  # Check if required columns exist in the dataframe
  required_columns <- c(y_name, continuous_name, categorical_name, treatment_name,
                        previous_status_name, subjectID_name)
  missing_columns <- setdiff(required_columns, names(df))

  if (length(missing_columns) > 0) {
    stop(paste("The following columns are missing from the dataframe:",
               paste(missing_columns, collapse = ", ")))
  }

  # Check if treatment variable is a factor
  if (!is.factor(df[[treatment_name]])) {
    stop("treatment_name must refer to a factor variable in the dataframe")
  }

  # Check each column to see if it is a factor
  is_factor_col <- sapply(df[, categorical_name, drop = FALSE], is.factor)
  if (!all(is_factor_col)) {
    # Find the names of the columns that failed the check
    non_factors <- names(df[, categorical_name, drop = FALSE])[!is_factor_col]

    # Stop execution and print a helpful error message
    stop(paste("Error: All categorical columns must be factors. The following columns are not:",
               paste(non_factors, collapse = ", ")))
  }

  Train_Data_Preparation_output <- Train_Data_Preparation(df = df,
                                                          y_name = y_name,
                                                          continuous_name = continuous_name,
                                                          categorical_name = categorical_name,
                                                          subjectID_name = subjectID_name)

  Y <- Train_Data_Preparation_output$Y
  X_continuous <- Train_Data_Preparation_output$X_continuous
  X_categorical <- Train_Data_Preparation_output$X_categorical

  ni_vector <- Train_Data_Preparation_output$ni_vector
  group_info <- Train_Data_Preparation_output$group_info

  outcome_model_results <- outcome_model(Y = Y,
                                         X_continuous = X_continuous,
                                         X_categorical = X_categorical,
                                         ni_vector = ni_vector,
                                         group_info = group_info,
                                         num_warmup = num_warmup,
                                         num_samples = num_samples,
                                         model_type = model_type,
                                         thin = thin,
                                         L = L,
                                         alpha = alpha,
                                         beta = beta,
                                         leaf_model_scale = leaf_model_scale,
                                         cutpoint_grid_size = cutpoint_grid_size,
                                         max_depth = max_depth,
                                         print_progress = print_progress,
                                         random_seed = random_seed)

  if (calculate_causal_estimand) {
    treatment_types <- levels(df[,treatment_name])
    causal_estimand_inference_list <- vector("list",length(treatment_types))
    names(causal_estimand_inference_list) <- treatment_types

    for (treatment_value in treatment_types) {
      if (print_progress) {
        message(paste("Calculate causal estimand for:", treatment_value))
      }
      causal_estimand_inference_list[[treatment_value]] <- causal_estimand_inference(outcome_model_results = outcome_model_results,
                                                                                     df = df,
                                                                                     continuous_name = continuous_name,
                                                                                     categorical_name = categorical_name,
                                                                                     treatment_name = treatment_name,
                                                                                     treatment_value = factor(treatment_value,
                                                                                                              levels = treatment_types),
                                                                                     previous_status_name = previous_status_name,
                                                                                     credible_interval_level = credible_interval_level,
                                                                                     num_of_grids = previous_status_grid_size)
    }

    output <- list(outcome_model_results = outcome_model_results,
                   causal_estimand = causal_estimand_inference_list,
                   df = df,
                   y_name = y_name,
                   continuous_name = continuous_name,
                   categorical_name = categorical_name,
                   treatment_name = treatment_name,
                   previous_status_name = previous_status_name,
                   subjectID_name = subjectID_name,
                   credible_interval_level = credible_interval_level)
  } else {

    output <- list(outcome_model_results = outcome_model_results,
                   df = df,
                   y_name = y_name,
                   continuous_name = continuous_name,
                   categorical_name = categorical_name,
                   treatment_name = treatment_name,
                   previous_status_name = previous_status_name,
                   subjectID_name = subjectID_name,
                   credible_interval_level = credible_interval_level)

  }

  return(output)

}

#' The random number generator for doubly censored response variable with the latent variable following normal assumption.
#'
#' @param n Sample size.
#' @param L0 The lower censoring limit.
#' @param R0 The upper censoring limit.
#' @param theta The location parameter of the latent variable.
#' @param sigma The standard deviation of the latent variable.
#'
#' @returns The simulated doubly censored response variable.
#' @noRd
#'
#' @examples
#'
#' set.seed(100)
#' y <- doubly_censored_rng(n = 1000, L0 = 0, R0 = 1, theta = 0.5, sigma = 0.2)
#' summary(y)
#' hist(y,breaks = 50)
#'
doubly_censored_rng <- function(n,L0,R0,theta,sigma) {
  Z <- rnorm(n = n, mean = 0.0, sd = sigma) + theta
  indexL <- (Z <= L0) + 0.0
  indexR <- (Z >= R0) + 0.0
  indexM <- (L0 < Z) * (Z < R0) + 0.0
  Y <- L0 * indexL + Z * indexM + R0 * indexR
  return(Y)
}

outcome_model_fixed_effects <- function(yij_previous,aij,X1j) {
  treatment_effects <- (-0.1)*(aij == "TREATMENT A") + (0.1)*(aij == "TREATMENT B")
  covariate_effects <- 0.2 * X1j
  previous_effects <- 0.1 * yij_previous + 0.05 * yij_previous^2
  treatment_interaction <- (aij == "TREATMENT A") * (-0.05*yij_previous) + (aij == "TREATMENT B") * (0.03 * yij_previous)

  output <- treatment_effects + treatment_interaction + covariate_effects + previous_effects
  return(output)
}

true_inner_expectation <- function(y_previous,treatment,confounder,sigma,
                                   random_effects) {
  fixed_effects <- outcome_model_fixed_effects(yij_previous = y_previous,
                                               aij = treatment,
                                               X1j = confounder)
  theta <- fixed_effects + random_effects
  output <- doubly_censored_expectation(theta = theta,
                                        sigma = sigma)
  return(output)
}

true_causal_estimand <- function(y_previous, treatment, confounder, sigma, sigma_u) {

  integrand <- function(u) {
    # Handle scalar input for non-vectorizable functions
    sapply(u, function(ui) {
      inner_result <- true_inner_expectation(y_previous = y_previous,
                                             treatment = treatment,
                                             confounder = confounder,
                                             sigma = sigma,
                                             random_effects = ui)
      output <- mean(inner_result) * dnorm(ui, mean = 0, sd = sigma_u)
      return(output)
    })
  }

  result <- integrate(integrand,
                      lower = -Inf,
                      upper = Inf,
                      rel.tol = .Machine$double.eps^0.5,
                      abs.tol = .Machine$double.eps^0.5)

  return(result$value)
}

#' @title Data Generation Program
#'
#' @description
#' Generates simulated data for evaluating the causal inference models.
#'
#' @param random_seed a single random seed for reproducibility. The datatype is \code{integer}.
#' @param N the total number of subjects to simulate. The datatype is \code{integer}.
#' @param sigma the global error standard deviation. The datatype is \code{double}.
#' @param sigma_u the standard deviation of the subject-level random effects. The datatype is \code{double}.
#'
#' @returns \code{data_generation} returns a simulated \code{data.frame}.
#' @export
#'
#' @example /inst/examples/data_generation.r
data_generation <- function(random_seed,N,sigma,sigma_u) {
  set.seed(random_seed)

  df_list <- vector("list", N)

  for (i in 1:N) {
    ni <- rpois(n = 1, lambda = 1) + 2

    # initilize output
    yi_current <- numeric(ni)
    yi_previous <- numeric(ni)
    ai <- character(ni)
    propensityi <- numeric(ni)

    # initial periodontal status for subjecti
    yi0 <- doubly_censored_rng(n = 1,
                               L0 = 0.0,
                               R0 = 1.0,
                               theta = 0.5,
                               sigma = sigma)
    yi_previous[1] <- yi0

    # covariates
    X1 <- rnorm(n = ni,mean = 0, sd = 0.1)

    # random effects
    Ui <- rnorm(n = 1, mean = 0, sd = sigma_u)

    for (j in 1:ni) {
      prob_eta <- yi_previous[j] + 0.5*X1[j]
      prob_treatment <- 1/(1+exp(-prob_eta))
      # treatment decision for patient i at initial visit
      aij <- sample(x = c("TREATMENT A",
                          "TREATMENT B"),
                    size = 1,
                    prob = c(prob_treatment,1-prob_treatment))
      ai[j] <- aij
      propensityi[j] <- prob_treatment
      fixed_effects <- outcome_model_fixed_effects(yij_previous = yi_previous[j],
                                                   aij = ai[j],
                                                   X1j = X1[j])

      yi_current[j] <- doubly_censored_rng(n = 1,
                                           L0 = 0.0,
                                           R0 = 1.0,
                                           theta = fixed_effects + Ui,
                                           sigma = sigma)

      if (j < ni) {
        yi_previous[j+1] <- yi_current[j]
      }
    }

    dfi <- data.frame(subjectID = paste0("subject",i),
                      current_value = yi_current,
                      previous_value = yi_previous,
                      treatment = ai,
                      confounder = X1,
                      propensity = propensityi)

    df_list[[i]] <- dfi

  }

  df1 <- do.call("rbind",df_list)

  df1$treatment <- factor(df1$treatment,
                          levels = c("TREATMENT A",
                                     "TREATMENT B"))

  df1$subjectID <- factor(df1$subjectID,
                          levels = unique(df1$subjectID))

  return(df1)
}

#' The 'BayesPocket' package.
#'
#' @description Implements a Bayesian double-censored model for causal inference in longitudinal studies of periodontal disease progression. The package provides tools for estimating causal effects of treatments on disease outcomes, accounting for time-varying confounders and left- and right-censored outcomes. It uses a Tobit regression model with extended Bayesian additive regression trees (XBART) for flexible modeling of complex relationships. The methodology is designed for observational dental data where treatments are assigned adaptively over time. Includes functions for model fitting and posterior inference of causal estimands.
#'
#' @docType package
#' @name BayesPocket
#' @importFrom stats dnorm integrate rnorm rpois pnorm quantile model.matrix rgamma
#' @importFrom GIGrvg rgig
#' @importFrom truncnorm rtruncnorm
#' @importFrom progress progress_bar
#' @importFrom stochtree createCppRNG createForest createForestDataset createForestModel createForestModelConfig createForestSamples createGlobalModelConfig createOutcome
#' @importFrom SoftBart Hypers MakeForest Opts
#' @importFrom pbmcapply pbmclapply
#' @importFrom parallel mclapply
#' @return This is the summary page. No return value.
"_PACKAGE"
#'
NULL

#' Prepare training set for causal inference
#'
#' @param df The input dataframe. The datatype is dataframe.
#' @param y_name  The name of the response variable. The datatype is character.
#' @param continuous_name The name of continuous predictors. The datatype is character.
#' @param categorical_name The name of categorical predictors. The datatype is character.
#' @param subjectID_name The name of subjectID. The datatype is character.
#' @noRd
#'
#' @returns A list of prepared training set.
Train_Data_Preparation <- function(df,
                                   y_name,
                                   continuous_name,
                                   categorical_name,
                                   subjectID_name) {

  # if df is a tibble, convert it as dataframe
  df <- as.data.frame(df)
  # define response variable
  Y <- as.numeric(df[,y_name])
  # define X_continuous
  X_continuous <- as.data.frame(df[continuous_name])
  # define X_categorical
  X_categorical <- as.data.frame(df[categorical_name])
  # define ni_vector
  ni_vector <- as.numeric(table(df[subjectID_name]))
  # define group_info
  group_info <- as.numeric(df[,subjectID_name])
  # define output
  output <- list(Y = Y,
                 X_continuous = X_continuous,
                 X_categorical = X_categorical,
                 ni_vector = ni_vector,
                 group_info = group_info)
  return(output)
}

generate_design_matrix <- function(X_continuous, X_categorical) {

  # 1. Check for NULL inputs
  if (is.null(X_continuous) || is.null(X_categorical)) {
    stop("Error: Neither X_continuous nor X_categorical can be NULL.")
  }

  # 2. Ensure inputs are data frames
  if (!is.data.frame(X_continuous) || !is.data.frame(X_categorical)) {
    stop("Error: Both X_continuous and X_categorical must be data frames.")
  }

  # 3. Combine into a single data frame
  X_combined <- cbind(X_continuous, X_categorical)

  # 4. Create the design matrix
  # "~ .^3" creates main effects, 2-way, and 3-way interactions.
  # "- 1" removes the intercept term.
  design_matrix <- model.matrix(~ .^3 - 1, data = X_combined)

  return(design_matrix)
}

#' The outcome model function. The outcome model assumes the noise follows a double-censored Tobit model with a subject-level random intercept.
#'
#' @param Y the outcome variable. The datatype is numeric.
#' @param X_continuous the dataframe with continuous variables. The datatype is data.frame.
#' @param X_categorical the dataframe with categorical variables. The datatype is data.frame.
#' @param ni_vector the numeric vector with the number of observations of each subject. The datatype is numeric.
#' @param group_info the numeric vector with the group information of each observation. The datatype is numeric.
#' @param num_warmup the number of warmup iterations. The datatype is integer.
#' @param num_samples the number of post-warmup iterations. The datatype is integer.
#' @param thin the period between saved samples. This should typically be left at its default (no thinning) unless memory is a problem.  The datatype is integer.
#' @param L the number of trees. The datatype is integer.
#' @param alpha the tree prior parameters. alpha*(1+depth)^(-beta) represents the prior probability of splitting at one of the cutpoints. Check equation (4) and related descriptions from "Stochastic Tree Ensembles for Regularized Nonlinear Regression" for more details. The datatype is double.
#' @param beta the tree prior parameters. alpha*(1+depth)^(-beta) represents the prior probability of splitting at one of the cutpoints. Check equation (4) and related descriptions from "Stochastic Tree Ensembles for Regularized Nonlinear Regression" for more details. The datatype is double.
#' @param leaf_model_scale the prior variance on leaf mean equals to \code{leaf_model_scale/L}. The datatype is double.
#' @param cutpoint_grid_size the number of cutoff points in XBART. The datatype is integer.
#' @param max_depth the maximum depth of tree allowed. The datatype is integer.
#' @param print_progress whether print progress bar or not. The datatype is boolean.
#' @param random_seed the random seed of the MCMC sampler. The datatype is integer.
#'
#' @returns A list of samples from the posterior distribution.
#' @noRd
outcome_model <- function(Y,
                          X_continuous,
                          X_categorical,
                          ni_vector, # the number of observations of each subjects.
                          group_info, # identification of observations belong to which subject.
                          num_warmup,
                          num_samples,
                          model_type,
                          thin,
                          L,
                          alpha,
                          beta,
                          leaf_model_scale,
                          cutpoint_grid_size,
                          max_depth = 10,
                          print_progress = TRUE,
                          random_seed = 100) {
  # set random seed ---------------------------------------------------------

  set.seed(random_seed)

  # define the number of subjects -------------------------------------------

  subject_size <- length(ni_vector)
  sample_size <- length(Y)

  indexL <- (Y == 0.0)
  indexM <- (0.0 < Y & Y < 1.0)
  indexR <- (Y == 1.0)

  sample_size_L <- sum(indexL)
  sample_size_R <- sum(indexR)

  YM <- Y[indexM]
  ZM <- YM
  Z <- Y

  # initialize values -------------------------------------------------------

  U_value <- rnorm(n = subject_size)
  random_effects <- rep(U_value, times = ni_vector)
  global_error_variance_value <- 1.0
  random_effects_variance_value <- 1.0
  fx_value <- rnorm(n = sample_size)

  if (model_type %in% c("Tobit-XBART","N-XBART")) {
    # combine covariate matrix
    # add the case that there is no categorical variable
    # for model validation purpose only
    if (is.null(X_categorical)) {
      Xall <- as.matrix(X_continuous)
    } else {
      X_categorical <- sapply(X_categorical, as.numeric)
      X_continuous <- as.matrix(X_continuous)
      Xall <- cbind(X_continuous,X_categorical)
    }

    # define feature types, 0 for numeric variable, 1 for categorical variable
    if (is.null(ncol(X_categorical))) {
      feature_types <- as.integer(rep(0,ncol(X_continuous)))
    } else {
      feature_types <- as.integer(c(rep(0,ncol(X_continuous)),
                                    rep(1,ncol(X_categorical))))
    }

    p_X <- ncol(Xall)
    variable_weights <- rep(1/p_X,p_X)

    # C++ dataset
    forest_dataset <- createForestDataset(Xall)

    # random number generator
    rng <- createCppRNG(random_seed)

    # sampling data structures
    forest_model_config <- createForestModelConfig(feature_types = feature_types,
                                                   num_trees = L,
                                                   num_features = p_X,
                                                   num_observations = sample_size,
                                                   variable_weights = variable_weights,
                                                   leaf_dimension = 1,
                                                   alpha = alpha,
                                                   beta = beta,
                                                   min_samples_leaf = 1,
                                                   max_depth = max_depth,
                                                   leaf_model_type = 0,
                                                   leaf_model_scale = leaf_model_scale,
                                                   variance_forest_shape = 1,
                                                   variance_forest_scale = 1,
                                                   cutpoint_grid_size = cutpoint_grid_size)

    global_model_config <- createGlobalModelConfig(global_error_variance = global_error_variance_value)

    forest_model <- createForestModel(forest_dataset = forest_dataset,
                                      forest_model_config = forest_model_config,
                                      global_model_config = global_model_config)

    # container of forest samples
    forest_samples <- createForestSamples(num_trees = L,
                                          leaf_dimension = 1,
                                          is_leaf_constant = TRUE,
                                          is_exponentiated = FALSE)

    # "active" forest state
    active_forest <- createForest(num_trees = L,
                                  leaf_dimension = 1,
                                  is_leaf_constant = TRUE,
                                  is_exponentiated = FALSE)

    # C++ outcome variable
    outcome <- createOutcome(outcome = Z)

    # Initialize the active forest and substract each root tree's predictions from outcome
    active_forest$prepare_for_sampler(dataset = forest_dataset,
                                      outcome = outcome,
                                      forest_model = forest_model,
                                      leaf_model_int = 0,
                                      leaf_value = 0.0)

    active_forest$adjust_residual(dataset = forest_dataset,
                                  outcome = outcome,
                                  forest_model = forest_model,
                                  requires_basis = FALSE,
                                  add = FALSE)

    rm(list = c("X_continuous","X_categorical","Xall")) # save memory
  } else if(model_type %in% c("Tobit-SBART","N-SBART")) {

    # 1. Prepare continuous matrix
    X_continuous <- as.matrix(X_continuous)
    num_continuous <- ncol(X_continuous)

    # 2. Create the continuous group IDs (starting at 1)
    group_cont <- 1:num_continuous

    # 3. Create the categorical matrix
    # Assuming X_categorical_df is your original dataframe of factors
    X_categorical <- model.matrix(~ . - 1, data = X_categorical)

    # 4. Extract the "assign" attribute and shift the IDs
    # assign_attr starts at 1. We shift it by num_continuous so it picks up
    # exactly where group_cont left off.
    assign_attr <- attr(X_categorical, "assign")
    group_cat <- assign_attr + num_continuous

    # 5. Combine matrices and group info
    Xall <- cbind(X_continuous, X_categorical)
    softforest_group_info <- c(group_cont, group_cat)

    Hypers_list <- Hypers(X = Xall,
                          Y = Z,
                          group = softforest_group_info,
                          alpha = alpha,
                          beta = beta,
                          k = 0.5/sqrt(leaf_model_scale * L),
                          sigma_hat = sqrt(global_error_variance_value),
                          num_tree = L,
                          normalize_Y = FALSE)

    Opts_list <- Opts(num_burn = num_warmup,
                      num_thin = thin,
                      num_save = num_samples/thin,
                      # do not print the chain's progress, pick a large number
                      num_print = 2147483647,
                      update_sigma_mu = FALSE,
                      update_s = FALSE,
                      update_alpha = FALSE,
                      update_beta = FALSE,
                      update_gamma = FALSE,
                      update_tau = FALSE,
                      update_tau_mean = FALSE,
                      update_sigma = FALSE,
                      cache_trees = TRUE
    )

    softforest <- MakeForest(hypers = Hypers_list,
                             opts = Opts_list,
                             warn = FALSE)

    rm(list = c("X_continuous","X_categorical")) # save memory
  } else if(model_type %in% c("Tobit-LH","N-LH")) {
    Xall <- generate_design_matrix(X_continuous = X_continuous,
                                   X_categorical = X_categorical)

    rm(list = c("X_continuous","X_categorical")) # save memory
  }


  # more initial value for horseshoe prior ----------------------------------

  if(model_type %in% c("Tobit-LH","N-LH")) {
    lambda2_value <- rep(1.0, ncol(Xall))
    tau2_value <- 1.0
    nu_value <- rep(1.0, ncol(Xall))
    xi_value <- 1.0
  }

  # define the output of the sampler ----------------------------------------

  MCMC_n <- num_warmup + num_samples

  # Calculate the number of samples after thinning
  num_samples_thinned <- num_samples / thin

  trees_outupt <- vector(mode = "list", length = num_samples_thinned)
  fx_output <- matrix(NA,
                      nrow = num_samples_thinned,
                      ncol = sample_size)
  global_error_variance_output <- numeric(num_samples_thinned)
  random_effects_variance_output <- numeric(num_samples_thinned)
  Z_output <- matrix(NA,
                     nrow = num_samples_thinned,
                     ncol = sample_size)
  random_effects_output <- matrix(NA,
                                  nrow = num_samples_thinned,
                                  ncol = sample_size)

  if(model_type %in% c("Tobit-LH","N-LH")) {
    beta_output <- matrix(NA,
                          nrow = num_samples_thinned,
                          ncol = ncol(Xall))
  }

  # begin the sampler -------------------------------------------------------

  if (print_progress) {
    pb <- progress_bar$new(total = MCMC_n,
                           format = "MCMC sampler [:bar] current: :current total: :total :percent eta: :eta",
                           width= 72)
  }

  # Initialize the Markov chain with num_warmup grow-from-root iterations
  gfr_flag <- TRUE
  for (i in 1:MCMC_n) {

    if (print_progress) {
      pb$tick()
    }

    # update latent variables -------------------------------------------------

    theta <- fx_value + random_effects

    # Tobit likelihood implies impute the latent variable Z
    if (model_type %in% c("Tobit-XBART","Tobit-SBART","Tobit-LH")) {
      if (sample_size_L > 0) {
        thetaL <- theta[indexL]
        ZL <- rtruncnorm(n = sample_size_L,
                         a = -Inf,
                         b = 0.0,
                         mean = thetaL,
                         sd = sqrt(global_error_variance_value))
        Z[indexL] <- ZL
      }

      if (sample_size_R > 0) {
        thetaR <- theta[indexR]
        ZR <- rtruncnorm(n = sample_size_R,
                         a = 1.0,
                         b = Inf,
                         mean = thetaR,
                         sd = sqrt(global_error_variance_value))
        Z[indexR] <- ZR
      }
    }

    # update random effects ---------------------------------------------------

    U_value <- update_U(Z = Z,
                        fixed_effects = fx_value,
                        ni_vector = ni_vector,
                        group_info = group_info,
                        global_error_variance = global_error_variance_value,
                        random_effects_variance = random_effects_variance_value)
    random_effects <- rep(U_value, times = ni_vector)

    if (model_type %in% c("Tobit-XBART","N-XBART")) {
      # propagate the newly sampled latent outcome to the BART model
      outcome$update_data(Z - random_effects)
      forest_model$propagate_residual_update(residual = outcome)
    }

    # update fixed effects ----------------------------------------------------


    if (model_type %in% c("Tobit-XBART","N-XBART")) {
      # use gfr for the first 100 iterations, decided on 2025-11-13
      if (i > 100) {
        gfr_flag <- FALSE
      }

      forest_model$sample_one_iteration(forest_dataset = forest_dataset,
                                        residual = outcome,
                                        forest_samples = forest_samples,
                                        active_forest = active_forest,
                                        rng = rng,
                                        forest_model_config = forest_model_config,
                                        global_model_config = global_model_config,
                                        keep_forest = TRUE,
                                        gfr = gfr_flag) # grow from root algorithm on/off

      fx_value <- forest_samples$predict_raw_single_forest(forest_dataset = forest_dataset,
                                                           forest_num = i - 1)
    } else if (model_type %in% c("Tobit-SBART","N-SBART")) {
      fx_value <-  softforest$do_gibbs(X = Xall,
                                       Y = Z - random_effects,
                                       X_test = Xall,
                                       i = 1)
    } else if (model_type %in% c("Tobit-LH", "N-LH")) {

      horseshoe_output <- horseshoe_iteration(y = Z - random_effects,
                                              X = Xall,
                                              sigma2 = global_error_variance_value,
                                              lambda2 = lambda2_value,
                                              tau2 = tau2_value,
                                              nu = nu_value,
                                              xi = xi_value)

      beta_value <- horseshoe_output$beta
      lambda2_value <- horseshoe_output$lambda2
      tau2_value <- horseshoe_output$tau2
      nu_value <- horseshoe_output$nu
      xi_value <- horseshoe_output$xi

      fx_value <- Xall %*% beta_value
    }
    fx_value <- as.numeric(fx_value)

    # update global error variance --------------------------------------------

    global_error_variance_value <- update_global_error_variance(Z = Z,
                                                                fixed_effects = fx_value,
                                                                random_effects = random_effects)
    if (model_type %in% c("Tobit-XBART", "N-XBART")) {
      global_model_config$update_global_error_variance(global_error_variance_value)
    } else if (model_type %in% c("Tobit-SBART", "N-SBART")) {
      softforest$set_sigma(x = sqrt(global_error_variance_value))
    }

    # update random effects variance ------------------------------------------

    random_effects_variance_value <- update_random_effects_variance(U_value = U_value,
                                                                    ni_vector = ni_vector)

    # Store samples after warmup period with thinning -------------------------
    if (i > num_warmup) {
      j <- i - num_warmup

      # Only store every 'thin'-th iteration
      if (j %% thin == 0) {
        sample_index <- j / thin  # Calculate the index in thinned arrays

        global_error_variance_output[sample_index] <- global_error_variance_value
        random_effects_variance_output[sample_index] <- random_effects_variance_value
        fx_output[sample_index,] <- fx_value
        Z_output[sample_index,] <- Z
        random_effects_output[sample_index,] <- random_effects

        if (model_type %in% c("Tobit-LH", "N-LH")) {
          beta_output[sample_index,] <- beta_value
        }
      }
    }
  }

  # 1. Create a base list with all the shared elements
  output <- list(
    global_error_variance_output   = global_error_variance_output,
    random_effects_variance_output = random_effects_variance_output,
    fx_output = fx_output,
    Z_output = Z_output,
    random_effects_output = random_effects_output,
    model_type = model_type,
    num_warmup = num_warmup,
    thin = thin
  )

  # 2. Append the specific element based on the model type
  if (model_type %in% c("Tobit-XBART","N-XBART")) {
    output$forest_samples <- forest_samples
  } else if (model_type %in% c("Tobit-SBART","N-SBART")) {
    output$softforest <- softforest
  } else if (model_type %in% c("Tobit-LH","N-LH")) {
    output$beta_output <- beta_output
  }

  return(output)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/main.R
\name{causal_inference_model}
\alias{causal_inference_model}
\title{Bayesian Mixed Treatment-State Causal Model (MTSCM)}
\usage{
causal_inference_model(
  df,
  y_name,
  continuous_name,
  categorical_name,
  treatment_name,
  previous_status_name,
  subjectID_name,
  num_warmup,
  num_samples,
  model_type,
  thin = 1,
  L = 50,
  alpha = 0.95,
  beta = 1.25,
  leaf_model_scale = 0.3/50,
  cutpoint_grid_size = 100,
  max_depth = 10,
  credible_interval_level = 0.95,
  print_progress = TRUE,
  random_seed = 100,
  calculate_causal_estimand = FALSE,
  previous_status_grid_size = 100
)
}
\arguments{
\item{df}{the input dataframe. The datatype is \code{data.frame}.}

\item{y_name}{the name of the response variable. The datatype is \code{character}.}

\item{continuous_name}{the name of continuous predictors. The datatype is \code{character}.}

\item{categorical_name}{the name of categorical predictors. The datatype is \code{character}.}

\item{treatment_name}{the name of the treatment predictor. The datatype is \code{character}.}

\item{previous_status_name}{the name of variable that represents previous status of a subject. The datatype is \code{character}.}

\item{subjectID_name}{the name of variable that represents subjectID. The datatype is \code{character}.}

\item{num_warmup}{the number of warmup iterations. The datatype is \code{integer}.}

\item{num_samples}{the number of post-warmup iterations. The datatype is \code{integer}.}

\item{model_type}{the type of causal inference models. It must be one of \code{"Tobit-XBART"}, \code{"Tobit-SBART"}, \code{"Tobit-LH"}, \code{"N-XBART"}, \code{"N-SBART"}, or \code{"N-LH"}.}

\item{thin}{the period between saved samples. This should typically be left at its default (no thinning) unless memory is a problem. The datatype is \code{integer}.}

\item{L}{the number of trees. The datatype is \code{integer}.}

\item{alpha}{the tree prior parameters. \code{alpha*(1+depth)^(-beta)} represents the prior probability of splitting at one of the cutpoints. Check equation (4) and related descriptions from "Stochastic Tree Ensembles for Regularized Nonlinear Regression" for more details. The datatype is \code{double}.}

\item{beta}{the tree prior parameters. \code{alpha*(1+depth)^(-beta)} represents the prior probability of splitting at one of the cutpoints. Check equation (4) and related descriptions from "Stochastic Tree Ensembles for Regularized Nonlinear Regression" for more details. The datatype is \code{double}.}

\item{leaf_model_scale}{the prior variance on leaf mean equals to \code{leaf_model_scale/L}. The datatype is \code{double}.}

\item{cutpoint_grid_size}{the number of cutoff points in XBART. The datatype is \code{integer}.}

\item{max_depth}{the maximum depth of tree allowed. The datatype is \code{integer}.}

\item{credible_interval_level}{the nominal level of credible intervals of causal estimand. The datatype is \code{double}.}

\item{print_progress}{whether print progress bar or not. The datatype is \code{boolean}.}

\item{random_seed}{the random seed of the MCMC sampler. The datatype is \code{integer}.}

\item{calculate_causal_estimand}{calculate causal estimand or not. The datatype is \code{boolean}.}

\item{previous_status_grid_size}{the number of cutoff points of previous status for causal estimands. The datatype is \code{integer}.}
}
\value{
a list containing posterior inference of causal estimands.
}
\description{
Fits a Bayesian Mixed Treatment-State Causal Model (MTSCM) tailored for longitudinal settings with irregular visits.
This model is specifically designed for bounded continuous outcomes with mass at both boundaries, such as the proportion of periodontal pockets exceeding 3 mm.
}
\details{
\strong{Outcome Model Description:}
The MTSCM utilizes a double-censored Tobit regression structure with subject-level random effects to capture within-subject correlation.
It uses tree-based ensemble priors (encompassing standard BART and Soft BART) to flexibly model complex, non-linear interactions without parametric restrictions.
Using a latent Gaussian variable \eqn{Z_{i,j+1}}, the data generating process for the outcome \eqn{Y_{i,j+1}} is formulated as:
\deqn{Z_{i,j+1} = f(A_{ij}, Y_{ij}, \mathbf{X}_{ij}) + U_i + \epsilon_{ij}, \quad \epsilon_{ij} \sim N(0, \sigma^2).}
\deqn{Y_{i,j+1} = \begin{cases} 0 & \text{if } Z_{i,j+1} \leq 0, \\ Z_{i,j+1} & \text{if } 0 < Z_{i,j+1} < 1, \\ 1 & \text{if } Z_{i,j+1} \geq 1. \end{cases}}

\strong{Causal Estimand Description:}
The framework estimates causal effects using the G-computation formula to marginalize conditional expectations over the covariate distribution.

\strong{Mixed Average Potential Outcome (MAPO):}
The MAPO represents the expected potential outcome across the population for a given treatment \eqn{a} and previous continuous disease state \eqn{y^{\star}}.
It is defined mathematically as:
\deqn{\theta(a,y^{\star}) = \frac{\sum_{i=1}^{N} \sum_{j=0}^{n_i - 1} \mathbb{E}(Y_{i,j+1} \mid A_{ij} = a, Y_{ij} = y^{\star}, \mathbf{X}_{ij} = \mathbf{x}_{ij})}{\sum_{i=1}^{N} n_{i}}.}

\strong{Mixed Probability of Disease Resolution (MPDR):}
The MPDR estimates the population-level probability of achieving a zero disease burden under a specific treatment \eqn{a} and previous disease state \eqn{y^{\star}}.
It is computed as:
\deqn{\theta^{(0)}(a,y^{\star}) = \frac{\sum_{i=1}^{N} \sum_{j=0}^{n_i - 1} \mathbb{P}(Y_{i,j+1} = 0 \mid A_{ij} = a, Y_{ij} = y^{\star}, \mathbf{X}_{ij} = \mathbf{x}_{ij})}{\sum_{i=1}^{N} n_{i}}.}
}
\examples{
df1 <- data_generation(random_seed = 100,
                       N = 100,
                       sigma = 0.2,
                       sigma_u = 0.1)

inference_output <- causal_inference_model(df = df1,
                                           y_name = "current_value",
                                           continuous_name = c("previous_value",
                                                               "confounder"),
                                           categorical_name = c("treatment"),
                                           treatment_name = "treatment",
                                           previous_status_name = "previous_value",
                                           subjectID_name = "subjectID",
                                           num_warmup = 2,
                                           num_samples = 2,
                                           model_type = "Tobit-XBART",
                                           thin = 1,
                                           L = 5,
                                           alpha = 0.95,
                                           beta = 1.25,
                                           leaf_model_scale = 0.3/5,
                                           cutpoint_grid_size = 100,
                                           max_depth = 10,
                                           credible_interval_level = 0.95,
                                           random_seed = 100,
                                           calculate_causal_estimand = FALSE,
                                           previous_status_grid_size = 2)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/causal_estimand_inference.R
\name{causal_estimand_inference}
\alias{causal_estimand_inference}
\title{Iterate Causal Estimand Calculations Over a Grid}
\usage{
causal_estimand_inference(
  outcome_model_results,
  df,
  continuous_name,
  categorical_name,
  treatment_name,
  treatment_value,
  previous_status_name,
  credible_interval_level = 0.95,
  num_of_grids = 50
)
}
\arguments{
\item{outcome_model_results}{the results from the outcome model. The datatype is \code{list}.}

\item{df}{the input dataframe. The datatype is \code{data.frame}.}

\item{continuous_name}{the name of continuous predictors. The datatype is \code{character}.}

\item{categorical_name}{the name of categorical predictors. The datatype is \code{character}.}

\item{treatment_name}{the name of the treatment predictor. The datatype is \code{character}.}

\item{treatment_value}{the value of the treatment variable. The datatype is \code{factor}.}

\item{previous_status_name}{the name of the variable that represents previous status. The datatype is \code{character}.}

\item{credible_interval_level}{the nominal level of credible intervals of causal estimand. The datatype is \code{double}.}

\item{num_of_grids}{the number of grid points in [0,1] to evaluate previous status on. The datatype is \code{integer}.}
}
\value{
a \code{list} of causal estimands calculated across the evaluation grid.
}
\description{
Wrapper to iterate causal estimand calculations over a grid of previous status values.
}
\details{
This function evaluates the causal estimands (such as MAPO and MPDR) across a specified grid of values for the previous disease state. For comprehensive details regarding the underlying framework, methodology, and the main model fitting procedure, please refer to \code{\link{causal_inference_model}}.
}
\examples{
# data generation ---------------------------------------------------------

df1 <- data_generation(random_seed = 100,
                       N = 100,
                       sigma = 0.2,
                       sigma_u = 0.1)

# draw samples from the posterior distribution ----------------------------

inference_output <- causal_inference_model(df = df1,
                                           y_name = "current_value",
                                           continuous_name = c("previous_value",
                                                               "confounder"),
                                           categorical_name = c("treatment"),
                                           treatment_name = "treatment",
                                           previous_status_name = "previous_value",
                                           subjectID_name = "subjectID",
                                           num_warmup = 2,
                                           num_samples = 2,
                                           model_type = "Tobit-XBART",
                                           thin = 1,
                                           L = 5,
                                           alpha = 0.95,
                                           beta = 1.25,
                                           leaf_model_scale = 0.3/5,
                                           cutpoint_grid_size = 100,
                                           max_depth = 10,
                                           credible_interval_level = 0.95,
                                           random_seed = 100,
                                           calculate_causal_estimand = FALSE,
                                           previous_status_grid_size = 2)

# calculate the causal estimand over a grid of previous status values -------

outcome_model_results <- inference_output$outcome_model_results

inference_results <- causal_estimand_inference(outcome_model_results = outcome_model_results,
                 df = inference_output$df,
                 continuous_name = inference_output$continuous_name,
                 categorical_name = inference_output$categorical_name,
                 treatment_name = inference_output$treatment_name,
                 treatment_value = factor("TREATMENT A",
                                          levels = levels(df1$treatment)),
                 previous_status_name = inference_output$previous_status_name,
                 credible_interval_level = 0.95,
                 num_of_grids = 2) # Example uses a small 2-point grid

# View the newly calculated closed-form causal estimands ------------------

# 1. Print results for the first grid point
cat("--- Results for Grid Point 1 ---\n")
print(inference_results[[1]]$mapo_summary)
print(inference_results[[1]]$mpdr_summary)

# 2. Print results for the second grid point
cat("\n--- Results for Grid Point 2 ---\n")
print(inference_results[[2]]$mapo_summary)
print(inference_results[[2]]$mpdr_summary)
}
\seealso{
\code{\link{causal_inference_model}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data_generation.R
\name{data_generation}
\alias{data_generation}
\title{Data Generation Program}
\usage{
data_generation(random_seed, N, sigma, sigma_u)
}
\arguments{
\item{random_seed}{a single random seed for reproducibility. The datatype is \code{integer}.}

\item{N}{the total number of subjects to simulate. The datatype is \code{integer}.}

\item{sigma}{the global error standard deviation. The datatype is \code{double}.}

\item{sigma_u}{the standard deviation of the subject-level random effects. The datatype is \code{double}.}
}
\value{
\code{data_generation} returns a simulated \code{data.frame}.
}
\description{
Generates simulated data for evaluating the causal inference models.
}
\examples{
df1 <- data_generation(random_seed = 100,
                       N = 100,
                       sigma = 0.2,
                       sigma_u = 0.1)

print(head(df1))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/BayesPocket-package.R
\docType{package}
\name{BayesPocket}
\alias{BayesPocket-package}
\alias{BayesPocket}
\title{The 'BayesPocket' package.}
\value{
This is the summary page. No return value.
}
\description{
Implements a Bayesian double-censored model for causal inference in longitudinal studies of periodontal disease progression. The package provides tools for estimating causal effects of treatments on disease outcomes, accounting for time-varying confounders and left- and right-censored outcomes. It uses a Tobit regression model with extended Bayesian additive regression trees (XBART) for flexible modeling of complex relationships. The methodology is designed for observational dental data where treatments are assigned adaptively over time. Includes functions for model fitting and posterior inference of causal estimands.
}
\author{
\strong{Maintainer}: Qingyang Liu \email{rh8liuqy@gmail.com} (\href{https://orcid.org/0000-0003-3265-6330}{ORCID})

Authors:
\itemize{
  \item Debdeep Pati \email{dpati2@wisc.edu}
  \item Yang Ni \email{yang.ni@austin.utexas.edu}
  \item Dipankar Bandyopadhyay \email{dbandyop@vcu.edu}
}

}
